@@ -1,5 +1,5 @@
 /* env - run a program in a modified environment
-   Copyright (C) 1986, 1991-2005, 2007-2008 Free Software Foundation, Inc.
+   Copyright (C) 1986, 1991-2005, 2007-2011 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,66 +16,6 @@
 
 /* Richard Mlynarik and David MacKenzie */
 
-/* Options:
-   -
-   -i
-   --ignore-environment
-	Construct a new environment from scratch; normally the
-	environment is inherited from the parent process, except as
-	modified by other options.
-
-   -u variable
-   --unset=variable
-	Unset variable VARIABLE (remove it from the environment).
-	If VARIABLE was not set, does nothing.
-
-   variable=value (an arg containing a "=" character)
-	Set the environment variable VARIABLE to value VALUE.  VALUE
-	may be of zero length ("variable=").  Setting a variable to a
-	zero-length value is different from unsetting it.
-
-   --
-	Indicate that the following argument is the program
-	to invoke.  This is necessary when the program's name
-	begins with "-" or contains a "=".
-
-   The first remaining argument specifies a program to invoke;
-   it is searched for according to the specification of the PATH
-   environment variable.  Any arguments following that are
-   passed as arguments to that program.
-
-   If no command name is specified following the environment
-   specifications, the resulting environment is printed.
-   This is like specifying a command name of "printenv".
-
-   Examples:
-
-   If the environment passed to "env" is
-	{ LOGNAME=rms EDITOR=emacs PATH=.:/gnubin:/hacks }
-
-   env - foo
-	runs "foo" in a null environment.
-
-   env foo
-	runs "foo" in the environment
-	{ LOGNAME=rms EDITOR=emacs PATH=.:/gnubin:/hacks }
-
-   env DISPLAY=gnu:0 nemacs
-	runs "nemacs" in the environment
-	{ LOGNAME=rms EDITOR=emacs PATH=.:/gnubin:/hacks DISPLAY=gnu:0 }
-
-   env - LOGNAME=foo /hacks/hack bar baz
-	runs the "hack" program on arguments "bar" and "baz" in an
-	environment in which the only variable is "LOGNAME".  Note that
-	the "-" option clears out the PATH variable, so one should be
-	careful to specify in which directory to find the program to
-	call.
-
-   env -u EDITOR LOGNAME=foo PATH=/energy -- e=mc2 bar baz
-	runs the program "/energy/e=mc2" with environment
-	{ LOGNAME=foo PATH=/energy }
-*/
-
 #include <config.h>
 #include <stdio.h>
 #include <sys/types.h>
@@ -83,6 +23,7 @@
 
 #include "system.h"
 #include "error.h"
+#include "quote.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "env"
@@ -91,13 +32,10 @@
   proper_name ("Richard Mlynarik"), \
   proper_name ("David MacKenzie")
 
-int putenv ();
-
-extern char **environ;
-
 static struct option const longopts[] =
 {
   {"ignore-environment", no_argument, NULL, 'i'},
+  {"null", no_argument, NULL, '0'},
   {"unset", required_argument, NULL, 'u'},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
@@ -109,17 +47,18 @@ usage (int status)
 {
   if (status != EXIT_SUCCESS)
     fprintf (stderr, _("Try `%s --help' for more information.\n"),
-	     program_name);
+             program_name);
   else
     {
       printf (_("\
 Usage: %s [OPTION]... [-] [NAME=VALUE]... [COMMAND [ARG]...]\n"),
-	      program_name);
+              program_name);
       fputs (_("\
 Set each NAME to VALUE in the environment and run COMMAND.\n\
 \n\
-  -i, --ignore-environment   start with an empty environment\n\
-  -u, --unset=NAME           remove variable from the environment\n\
+  -i, --ignore-environment  start with an empty environment\n\
+  -0, --null           end each output line with 0 byte rather than newline\n\
+  -u, --unset=NAME     remove variable from the environment\n\
 "), stdout);
       fputs (HELP_OPTION_DESCRIPTION, stdout);
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
@@ -127,7 +66,7 @@ Set each NAME to VALUE in the environment and run COMMAND.\n\
 \n\
 A mere - implies -i.  If no COMMAND, print the resulting environment.\n\
 "), stdout);
-      emit_bug_reporting_address ();
+      emit_ancillary_info ();
     }
   exit (status);
 }
@@ -137,6 +76,7 @@ main (int argc, char **argv)
 {
   int optc;
   bool ignore_environment = false;
+  bool opt_nul_terminate_output = false;
 
   initialize_main (&argc, &argv);
   set_program_name (argv[0]);
@@ -144,23 +84,26 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  initialize_exit_failure (EXIT_FAILURE);
+  initialize_exit_failure (EXIT_CANCELED);
   atexit (close_stdout);
 
-  while ((optc = getopt_long (argc, argv, "+iu:", longopts, NULL)) != -1)
+  while ((optc = getopt_long (argc, argv, "+iu:0", longopts, NULL)) != -1)
     {
       switch (optc)
-	{
-	case 'i':
-	  ignore_environment = true;
-	  break;
-	case 'u':
-	  break;
-	case_GETOPT_HELP_CHAR;
-	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
-	default:
-	  usage (EXIT_FAILURE);
-	}
+        {
+        case 'i':
+          ignore_environment = true;
+          break;
+        case 'u':
+          break;
+        case '0':
+          opt_nul_terminate_output = true;
+          break;
+        case_GETOPT_HELP_CHAR;
+        case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+        default:
+          usage (EXIT_CANCELED);
+        }
     }
 
   if (optind < argc && STREQ (argv[optind], "-"))
@@ -173,25 +116,36 @@ main (int argc, char **argv)
     }
 
   optind = 0;			/* Force GNU getopt to re-initialize. */
-  while ((optc = getopt_long (argc, argv, "+iu:", longopts, NULL)) != -1)
-    if (optc == 'u')
-      putenv (optarg);		/* Requires GNU putenv. */
+  while ((optc = getopt_long (argc, argv, "+iu:0", longopts, NULL)) != -1)
+    if (optc == 'u' && unsetenv (optarg))
+      error (EXIT_CANCELED, errno, _("cannot unset %s"), quote (optarg));
 
   if (optind < argc && STREQ (argv[optind], "-"))
     ++optind;
 
   while (optind < argc && strchr (argv[optind], '='))
-    putenv (argv[optind++]);
+    if (putenv (argv[optind++]))
+      {
+        char *name = argv[optind - 1];
+        *(strchr (name, '=')) = '\0';
+        error (EXIT_CANCELED, errno, _("cannot set %s"), quote (name));
+      }
 
   /* If no program is specified, print the environment and exit. */
   if (argc <= optind)
     {
       char *const *e = environ;
       while (*e)
-	puts (*e++);
+        printf ("%s%c", *e++, opt_nul_terminate_output ? '\0' : '\n');
       exit (EXIT_SUCCESS);
     }
 
+  if (opt_nul_terminate_output)
+    {
+      error (0, errno, _("cannot specify --null (-0) with command"));
+      usage (EXIT_CANCELED);
+    }
+
   execvp (argv[optind], &argv[optind]);
 
   {
