@@ -1,10 +1,11 @@
 /* sleep - delay for a specified amount of time.
-   Copyright (C) 84, 1991-1997, 1999-2004 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1991-1997, 1999-2005, 2007-2011 Free Software
+   Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,12 +13,10 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include <config.h>
 #include <stdio.h>
-#include <assert.h>
 #include <sys/types.h>
 #include <getopt.h>
 
@@ -25,23 +24,23 @@
 #include "c-strtod.h"
 #include "error.h"
 #include "long-options.h"
+#include "quote.h"
 #include "xnanosleep.h"
 #include "xstrtod.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "sleep"
 
-#define AUTHORS "Jim Meyering", "Paul Eggert"
-
-/* The name by which this program was run. */
-char *program_name;
+#define AUTHORS \
+  proper_name ("Jim Meyering"), \
+  proper_name ("Paul Eggert")
 
 void
 usage (int status)
 {
   if (status != EXIT_SUCCESS)
     fprintf (stderr, _("Try `%s --help' for more information.\n"),
-	     program_name);
+             program_name);
   else
     {
       printf (_("\
@@ -50,13 +49,14 @@ Usage: %s NUMBER[SUFFIX]...\n\
 Pause for NUMBER seconds.  SUFFIX may be `s' for seconds (the default),\n\
 `m' for minutes, `h' for hours or `d' for days.  Unlike most implementations\n\
 that require NUMBER be an integer, here NUMBER may be an arbitrary floating\n\
-point number.\n\
+point number.  Given two or more arguments, pause for the amount of time\n\
+specified by the sum of their values.\n\
 \n\
 "),
-	      program_name, program_name);
+              program_name, program_name);
       fputs (HELP_OPTION_DESCRIPTION, stdout);
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
-      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+      emit_ancillary_info ();
     }
   exit (status);
 }
@@ -104,15 +104,15 @@ main (int argc, char **argv)
   bool ok = true;
 
   initialize_main (&argc, &argv);
-  program_name = argv[0];
+  set_program_name (argv[0]);
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
   atexit (close_stdout);
 
-  parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
-		      usage, AUTHORS, (char const *) NULL);
+  parse_long_options (argc, argv, PROGRAM_NAME, PACKAGE_NAME, Version,
+                      usage, AUTHORS, (char const *) NULL);
   if (getopt_long (argc, argv, "", NULL, NULL) != -1)
     usage (EXIT_FAILURE);
 
@@ -127,16 +127,16 @@ main (int argc, char **argv)
       double s;
       const char *p;
       if (! xstrtod (argv[i], &p, &s, c_strtod)
-	  /* Nonnegative interval.  */
-	  || ! (0 <= s)
-	  /* No extra chars after the number and an optional s,m,h,d char.  */
-	  || (*p && *(p+1))
-	  /* Check any suffix char and update S based on the suffix.  */
-	  || ! apply_suffix (&s, *p))
-	{
-	  error (0, 0, _("invalid time interval `%s'"), argv[i]);
-	  ok = false;
-	}
+          /* Nonnegative interval.  */
+          || ! (0 <= s)
+          /* No extra chars after the number and an optional s,m,h,d char.  */
+          || (*p && *(p+1))
+          /* Check any suffix char and update S based on the suffix.  */
+          || ! apply_suffix (&s, *p))
+        {
+          error (0, 0, _("invalid time interval %s"), quote (argv[i]));
+          ok = false;
+        }
 
       seconds += s;
     }
