@@ -1,5 +1,5 @@
 /* GNU's uptime.
-   Copyright (C) 1992-2002, 2004-2010 Free Software Foundation, Inc.
+   Copyright (C) 1992-2002, 2004-2011 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -151,11 +151,7 @@ print_uptime (size_t n, const STRUCT_UTMP *this)
   printf (ngettext ("%lu user", "%lu users", entries),
           (unsigned long int) entries);
 
-#if defined HAVE_GETLOADAVG || defined C_GETLOADAVG
   loads = getloadavg (avg, 3);
-#else
-  loads = -1;
-#endif
 
   if (loads == -1)
     putchar ('\n');
